class Conference < ApplicationRecord
  has_many :sessions
  has_many :speakers
  has_many :stages
  has_many :revision_sets

  serialize :data, coder: JSON

  validates :time_zone, presence: true, inclusion: { in: ActiveSupport::TimeZone.all.map(&:name) }

  has_many :relevant_stage_links, class_name: 'RelevantStage'
  has_many :relevant_stages, through: :relevant_stage_links, source: :stage

  def days
    (starts_at.to_date..ends_at.to_date)
  end

  def to_param
    slug
  end

  def starts_at_in_local_time
    starts_at.in_time_zone(time_zone || 'UTC')
  end

  def ends_at_in_local_time
    ends_at.in_time_zone(time_zone || 'UTC')
  end

  def schedule_url
    data['schedule_url']
  end

  def filedrop_url
    data['filedrop_url']
  end

  def engelsystem_url
    data['engelsystem_url']
  end

  def heartbeat_url
    data['heartbeat_url']
  end

  def fetch_translation_angel_id
    fetch_engelsystem("angeltypes")
      &.find { |t| t['name'] == 'Translation Angel' }
      &.dig('id')
  end

  def fetch_engelsystem(endpoint)
    begin
      endpoint_url = engelsystem_url + endpoint
      Rails.logger.debug("Querying engelsystem API at #{endpoint_url}")
      response = HTTParty.get(
        endpoint_url,
        headers: {
          'Accept' => 'application/json',
          "x-api-key" => fetch_credential("engelsystem_token")
        },
        timeout: 10
      )
      return response.success? ? JSON.parse(response.body)["data"] : nil
    rescue => e
      Rails.logger.warn("Engelsystem query for #{endpoint} failed: #{e.message}")
      return nil
    end
  end
end
